"use strict";
define(function (require, exports, module) {
    /**
     * 解决事件监听 this 问题，这里 转接一下
     */
    class MyEventListener {

        constructor() {
            //resize
            window.addEventListener(('orientationchange' in window ? 'orientationchange' : 'resize'), this.RSizeListener, false);
            window.addEventListener('pageshow', this.RSizeListener, false);
            window.addEventListener('DOMContentLoaded', this.RSizeListener, false);
            /**
             * 窗口事件.
             * js中的键盘事件只有三种：keydown、keyup、keypress。
             * 它们触发的顺序是：keydown -> keypress -> keyup。
             * 当按下一个键不放开，一般会重复地触发 keydown+keypress，直到放开后触发一个 keyup 事件。
             */
            window.addEventListener('keydown', this.KeydownListener, false);
            //keyup
            window.addEventListener('keyup', this.KeyUpListener, false);
            // window.addEventListener('DOMsubtreeModified', function (e) {
            //     console.log("DOMsubtreeModified : ", e)
            // });

            let mutationHandle = require("./impl/MutationObserverImpl")
            // 并将回调函数传给它
            let MutationObserver = window.MutationObserver || window.WebKitMutationObserver;
            let observer = new MutationObserver(function (mutationList, observer) {
                mutationHandle.handle(mutationList);
            });

            // 在 MutationObserver 实例上调用 `observe` 方法，
            // 并将要观察的元素与选项传给此方法
            observer.observe(document.getElementById("yxl_note"),
                {
                    // 当元素的属性变化时触发回调（默认为 false）。
                    attributes: true,
                    //当目标节点的子节点被添加或删除时触发回调（默认为 false）。
                    childList: true,
                    // 当设置为 true 时，监视目标节点及其所有后代节点的变化（默认为 false）。
                    subtree: true,
                    // 当节点的文本内容变化时触发回调（默认为 false）。
                    characterData: true,
                    //当属性变化时，记录变化前的属性值（默认为 false）。
                    attributeOldValue: true,
                    // 当文本节点变化时，记录变化前的文本内容（默认为 false）。
                    characterDataOldValue: true
                });

            //中文输入
            document.getElementById("yxl_note").addEventListener('compositionstart',
                this.CompositionstartListener);
            document.getElementById("yxl_note").addEventListener('compositionend',
                this.CompositionendListener);

            // 注册光标移动事件
            // document.getElementById("yxl_note").addEventListener('selectionchange', function (e) {
            //     console.log("注册光标移动事件", window.Selection)
            // }, false);


            //这里监听鼠标按下事件
            document.getElementById("_style_utils").addEventListener("mousedown", this.Mousedown, false);

            //样式事件监听
            let styleList = document.getElementsByClassName("fixStyleInnerSpan");
            // console.log("styleList : ", styleList);
            if (styleList && styleList.length > 0) {
                for (let i = 0; i < styleList.length; i++) {
                    // console.log(styleList[i]);
                    styleList[i].addEventListener('click', this.ApplyStyleListener, false);
                }
            }



            //::before
            //样式事件监听
            let myBeforeStyleList = document.getElementsByClassName("my_before_style");
            // console.log("styleList : ", styleList);
            if (myBeforeStyleList && myBeforeStyleList.length > 0) {
                for (let i = 0; i < myBeforeStyleList.length; i++) {
                    myBeforeStyleList[i].addEventListener('click', this.ApplyMyBeforeStyleListener, false);
                }
            }

            //监听下拉事件
            let dropStyleList = document.querySelectorAll(".my_style_nav ul");
            for (let i = 0; i < dropStyleList.length; i++) {
                dropStyleList[i].addEventListener("click", this.DropDownStyleSelectListener, false);
            }
        }

        DropDownStyleSelectListener(e) {
            require('./impl/DropDownStyleSelectEventImpl').handle(e);
        }


        Mousedown(e) {
            const handle = require('./impl/MouseDownEventImpl');
            handle.handle(e);
        }


        /**
         * 使用样式事件
         * @param e
         * @constructor
         */
        ApplyStyleListener(e) {
            const handle = require('./impl/ApplyStyleEventImpl');
            handle.handle(e);
        }

        /**
         * :: before 样式
         * @param e
         * @constructor
         */
        ApplyMyBeforeStyleListener(e) {
            const handle = require('./impl/BeforeStyleEventImpl');
            handle.handle(e);
        }

        /**
         *
         * @param {*} e
         */
        KeydownListener(e) {
            const keyDownHandle = require('./impl/KeyDownEventImpl');
            keyDownHandle.handle(e);
        }

        /**
         *
         * @param e
         * @constructor
         */
        KeyUpListener(e) {
            const keyUpHandle = require('./impl/KeyUpEventImpl');
            keyUpHandle.handle(e);
        }

        /**
         * 输入事件
         * @param e
         * @constructor
         */
        InputListener(e) {
            const handle = require('./impl/InputEventImpl');
            handle.handle(e);
        }

        /**
         * 中文输入开始事件
         * @param e
         * @constructor
         */
        CompositionstartListener(e) {
            const ctx = require('../common/ctx');
            ctx.Compositionstart = true
        }

        /**
         * 中文输入结束事件
         * @param e
         * @constructor
         */
        CompositionendListener(e) {
            const ctx = require('../common/ctx');
            ctx.Compositionstart = false
        }

        /**
         * 尺寸变化事件
         * @param e
         * @constructor
         */
        RSizeListener(e) {
            // console.log("RSizeListener : ", e);
            let handle = require("./impl/RSizeEventImpl");
            handle.handle(e);
        }


    }

    // 初始化一次
    module.exports = new MyEventListener();
});

